package websocket

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"strings"

	"github.com/gorilla/websocket"
	"github.com/leskraas/dino/synth"
	// "github.com/leskraas/dino/pkg/fileManager"
)

type Client struct {
	ID   string
	Conn *websocket.Conn
	Pool *Pool
}

type Message struct {
	Type int  `json:"type"`
	Data Data `json:"data"`
}

type Data struct {
	Type     string `json:"type"`
	Content  string `json:"content"`
	FileName string `json:"filename"`
}

// func executeYosys(path string) {
// 	app := "../yosys/yosys"
// 	// arg0 := []string{"-p", "prep -auto-top -ifx; write_json ../files/out.json", path}
// 	arg0 := []string{"-p", "prep -auto-top -flatten; write_json ../files/out.json", path}
// 	yosysCmd := exec.Command(app, arg0...) //, arg1, arg2)

// 	err := yosysCmd.Run()
// 	if err != nil {
// 		// log.Fatalf("yosysCmd.Run() failed with '%s'\n", err)
// 		log.Printf("yosysCmd.Run() failed with '%s'\n", err)
// 	}

// }

// func executeNetlistsvg() {
// 	app := "node"
// 	arg := []string{"../netlistsvg/bin/netlistsvg", "../files/out.json", "-o", "../files/out.svg"}
// 	netlistsvgCmd := exec.Command(app, arg...)

// 	err := netlistsvgCmd.Run()
// 	if err != nil {
// 		// log.Fatalf("netlistsvgCmd.Run() failed with '%s'\n", err)
// 		log.Printf("netlistsvgCmd.Run() failed with '%s'\n", err)
// 	}
// }

func (c *Client) Read() {
	defer func() {
		c.Pool.Unregister <- c
		c.Conn.Close()
	}()

	for {
		messageType, p, err := c.Conn.ReadMessage()
		if err != nil {
			log.Println(err)
			return
		}

		var d Data
		if err := json.Unmarshal(p, &d); err != nil {
			panic(err)
		}
		message := Message{Type: messageType, Data: d}

		if d.Type == "oppstart" {
			filepath := "../files/user/."
			files, _ := ioutil.ReadDir(filepath)
			message.Data.Type = "oppstart"
			var allFiles []string
			for _, file := range files {
				fileString := string(file.Name())
				fileString = strings.Replace(fileString, ".v", "", -1)
				allFiles = append(allFiles, fileString)
			}
			allFilesString, _ := json.Marshal(allFiles)
			message.Data.Content = string(allFilesString)

		}
		if d.Type == "save" {
			filepath := "../files/user/" + d.FileName + ".v"
			ioutil.WriteFile(filepath, []byte(d.Content), 0644)
			message.Data.Type = "save"

		}

		if d.Type == "verify" {
			// fileManager.WriteToFile("../files/ny", d.Content)
			// executeYosys("../files/ny.v")
			// executeNetlistsvg()
			// filepath := "../files/" + d.Content + "/" + d.FileName
			// fmt.Printf("_-------------: %s", d.Content)
			out, err := synth.CodeCheck(d.FileName, d.Content)
			// fmt.Printf("error CodeCheck: %s\n", err.Error())
			if err != nil {
				message.Data.Type = "errormsg"
				message.Data.Content = out + "\n " + string(err.Error())
				c.Pool.Broadcast <- message
				message.Data.Type = "svg"
				message.Data.Content = ""
			} else {
				message.Data.Type = "errormsg"
				message.Data.Content = "No error in code"
				c.Pool.Broadcast <- message
				synth.NetlistSVG(d.FileName)
				synth.RunSynth(d.FileName, d.Content)
				svgfilepath := "../svg/" + d.FileName
				file, _ := os.Open("../svg/" + svgfilepath + ".svg")
				svg, _ := ioutil.ReadAll(file)
				message.Data.Type = "svg"
				message.Data.Content = string(svg)
			}
			// synth.ProgDevice(d.FileName)

		}

		if d.Type == "prog" {
			synth.ProgDevice(d.FileName)
		}

		if d.Type == "newfile" {
			filename := "../filer/user/" + d.FileName + ".v"

			// _, err := os.Create(filename)
			os.Create(filename)
			ioutil.WriteFile(filename, []byte(d.Content), 0644)
			message.Data.Type = "newfile"
			if d.Content == "" {
				file, _ := os.Open("../verilog/templet.v")
				content, _ := ioutil.ReadAll(file)
				message.Data.Content = string(content)
			}
			message.Data.FileName = d.FileName
			// if err != nil {

			// 	message.Data.Type = "newfileerror"
			// 	message.Data.Content = err.Error()
			// }
		}

		if d.Type == "getfilecontent" {
			fileFolder := "../files/" + d.Content
			filename := fileFolder + "/" + d.FileName + ".v"

			file, err := os.Open(filename)
			fmt.Println(err)
			content, _ := ioutil.ReadAll(file)
			fmt.Println(err)
			// fmt.Println(string(content))
			message.Data.Type = "filecontent"
			message.Data.Content = string(content)
			message.Data.FileName = d.FileName
			// if err != nil {

			// 	message.Data.Type = "newfileerror"
			// 	message.Data.Content = err.Error()
			// }
		}
		// fmt.Printf("d.Type: %s,   d.Content: %s,    d.FileName: %s \n ", d.Type, d.Content, d.FileName)
		fmt.Println(message)
		c.Pool.Broadcast <- message

		// fmt.Printf("Message Received: %+v\n", d.Content)
	}
}
